-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

-- Create Functua Namespace.
local functua = { }

-- Expose Functua as namespace in npcg to avoid conflicts later.
npcg.functua  = functua

-------------------------------------------------------------------------------------
-- | Prelude | ----------------------------------------------------------------------
-------------------------------------------------------------------------------------

-- id :: a -> a
-- Returns the input.
function functua.id(x)
  return x
end

-- replicate :: int -> a -> { a }
-- Returns a list of length n where every element is x.
function functua.replicate(n, x)
	local output = { }
	for i = 1, n do table.insert(output, x) end
	return output
end

-- iterate :: (a -> ()) -> int -> int -> ()
-- Performs a certain action incrementally, starting from x, then x+1 until (inclusive) n.
function functua.iterate(action, x, n)
  for i = x, n do
    action(i)
  end
end

-- const :: a -> (b -> a)
-- Returns a function that always returns x.
function functua.const(x)
  return function(...) return x end
end

-- invokeAll :: { (() -> ()) } -> ()
-- Invokes all actions in a list.
function functua.invokeAll(xs)
  functua.forEach(function(x) x() end, xs)
end


-- mergeTables :: { a } -> { a } -> { a }
--[[ Merges an arbitrary amount of arrays into one.
-- Ensures:
-- 1) ({1},{2}) = {1,2}
-- 2) ({ },{2}) = {  2}
-- 3) ({1},{ }) = {  1}
-- 4) (nil,{2}) = {  2}
-- 5) ({1},nil) = {  1} ]]
function functua.mergeTables(...)
  --print("mergeTables called!")
	local output = { }
	for _,t in pairs({...}) do  
		for _,v in pairs(t or { }) do -- <-- only takes values, indicating it's a list, but also uses pairs, which indicates it should be a table..
			table.insert(output, v)
		end
	end
	return output
end

function functua.mergeTables2(...)
  --print("mergeTables2 called!")
  local output = { }
  for _,t in ipairs({...}) do
    for k,v in pairs(t or { }) do
      output[k] = v
    end
  end
  return output
end

function functua.mergeTablesWithoutDuplicates(...)
  --print("mergeTablesWithoutDuplicates called!")
  local output = { }
	for _,t in pairs({...}) do  -- <-- only takes values, indicating it's a list, but also uses pairs, which indicates it should be a table..
		for _,v in pairs(t or { }) do
			if not functua.table.contains(v, output) then
        table.insert(output, v)
      end
		end
	end
	return output
end

function functua.mergeTablesWithoutDuplicates2(...)
  --print("mergeTablesWithoutDuplicates2 called!")
  local output = { }
	for _,t in pairs({...}) do  -- <-- only takes values, indicating it's a list, but also uses pairs, which indicates it should be a table..
		for k,v in pairs(t or { }) do
			--if not contains(k, output) then
        output[k] = v
      --end
		end
	end
	return output
end


-- shift :: { k, v } -> { { key :: k, value :: v } }
-- Returns a list of key-value-pairs from a table of key-value-pairs.
-- Note: The original k is now in x.key and the original v is now in x.value.
-- Possibly Redundant function.
function functua.shift(xs)
  print("Functua Output: functua.shift called!")
  return functua.table.shift(xs)
end

-- unshift :: { { key :: k, value :: v } } -> { k, v }
-- Returns a table list of key-value-pairs.
-- Possibly Redundant function.
function functua.unshift(xs)
	print("Functua Output: functua.unshift called!")
  return functua.table.unshift(xs)
end



-- Possibly Redundant function.
function functua.equals(x, y)
  print("Functua Output: functua.equals called!")
  return x == y
end

-- Possibly Redundant function.
function functua.composeEqualsFunction(y)
  print("Functua Output: functua.composeEqualsFunction called!")
  return function(x) return x == y end
end


-- both :: (a -> bool) -> (a -> bool) -> (a -> bool)
-- Returns a function that checks both predicates using a single value.
function functua.both(p, q)
  return function(x) return p(x) and q(x) end
end

-- Possibly Redundant function.
function functua.bothL(p, q)
  return function(...) return p(...) and q(...) end
end

-- End of File.