-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua = npcg.functua

-- Create Random Namespace.
local myRandom = { }

-- Expose to functua namespace.
functua.random = myRandom

-------------------------------------------------------------------------------------
-- | Random | -----------------------------------------------------------------------
-------------------------------------------------------------------------------------

-- randomFrom :: { a } -> a
-- Returns a random element from a list.
-- Implicitly, if list is empty, nil will be returned.
function myRandom.randomFrom(xs)
	local c = functua.table.count(xs)
	for _,v in pairs(xs) do
		local rand = random(1, c)
		if rand == 1 then
			return v 
		else
			c = c - 1
		end
	end
end

-- weightedRandomFrom :: { a } -> (a -> int) -> a
-- Returns a random element from a list using a weight function.
-- Implicitly, if the list is empty, nil will be returned.
function myRandom.weightedRandomFrom(list, weightFunction)
  local sumOfWeights = functua.table.sumBy(list, weightFunction)
  if sumOfWeights == 0 then return nil end

  -- If we consider items to be duplicated by their weight in a fictive list we can take the item at the random index.
  local randomIndex = random(1, sumOfWeights)
  
  local foundKey = functua.table.findKey(
    function(x)
      if    x >= randomIndex  then  return true
      else                    randomIndex = randomIndex - x
                              return false end
    end,
    functua.table.map(weightFunction, list)
  )
    
  return list[foundKey]
end

-- uniqueRandomsFrom :: int -> { a } -> { a }
-- Returns a number between 1 and maxReturns of random elements from a list.
function myRandom.uniqueRandomsFrom(maxReturns, list)
  if maxReturns <= 0 then return { } end
  
  local output = { }
	local c = functua.count(list)
	for _,v in pairs(list) do
		local rand = random(1, c)
		if rand == 1 then
			table.insert(output, v) -- This line is changed to hopefully fix the bug >.>
      maxReturns = maxReturns - 1
      if maxReturns <= 0 then
        return output
      end
		else  -- Else clause jumbles up the c variable, this guarantees that there could be less returns than maxReturns.
			c = c - 1
		end
	end
  return output
end

-- uniqueRandomsFromCartesian :: { a } -> { b } -> a,b
-- Returns a random element of the first list, and one of the second list, ensuring they are not the same.
-- (x, y) | x <- xs and y <- ys and x ~= y 
function myRandom.uniqueRandomsFromCartesian(xs, ys)
  local x = myRandom.randomFrom(xs)
  local y = myRandom.randomFrom(ys)
  while x == y and y ~= nil do -- This can loop infinitely.
    y = myRandom.randomFrom(ys)
  end
  return x, y
end

-- eof