-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua 	= npcg.functua
local util 			= npcg.functua.util
local replicate = functua.replicate

local generator = { }
local data = { }
local tests = { }

-- generators and data do not exist yet
npcg.generators.motivation	= generator
npcg.data.motivation				= data
npcg.tests.motivation				= tests

local motivations = { }

local function default(replacements)
  replacements = replacements or { }
  return { 
    races       = replacements[     "races"] or { "any" },
    classes     = replacements[   "classes"] or { "any" },
    faiths      = replacements[    "faiths"] or { "any" },
    genders     = replacements[   "genders"] or { "any" },
    alignments  = replacements["alignments"] or { "any" },
    ages        = replacements[      "ages"] or npcg.data.age.olderThan("child"),
		tags				= replacements[      "tags"] or {       },
		weight			= 1,
  }
end

motivations = {
-- a ---------------------------------------------------------------------
[  "aquiring an ancient artifact"                      ] = default({

}),
[  "attending a prayer for the Holy Light"             ] = default({
  faiths = { 
		"church of the holy light" },
  alignments = npcg.data.alignment.alignmentsByEthics(
		"lawful")
}),  
[  "atoning for previous wrongdoings"                  ] = default({
  alignments  = { 
		"neutral good",
		"true neutral" },
}),   
[  "avenging %xir brother"                             ] = default({
  alignments  = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
}),
[  "avenging %xir sister"                              ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
}),
[  "avenging the death of %xir parents"                ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "avenging the death of %xir teddybear"              ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
	ages = npcg.data.age.agesBetween(
		"child",
		"child")
}),
[  "avenging the attack of %xir village"               ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "avenging %xir spouse"                              ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "avenging %xir people"                              ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "avenging the attack on Silvermoon"                 ] = default({
  races = {
		"blood elf" },
  alignments = npcg.data.alignment.getAlignmentsByTag("vengeful"),
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "avenging the death of %xir leader"                 ] = default({
  alignments = npcg.data.alignment.getAlignmentsByTag(
		"vengeful"),
	ages = npcg.data.age.olderThan(
		"teen")
}),

-- b ---------------------------------------------------------------------
[  "badly wants to be admired"                         ] = default({
  alignments = {
		"chaotic good",
		"chaotic neutral",
		"lawful evil",
		"neutral evil",
		"chaotic evil" },
}),
[  "baking a cake"                                     ] = default({

}),
[  "beating a HearthStone Adventure"                   ] = default({

}),
[  "becoming a %randomClass"                           ] = default({

}),
[  "becoming leader of %xir religion"                  ] = default({
  alignments = {
		"lawful good",
		"lawful neutral",
		"lawful evil" },
	ages = npcg.data.age.olderThan(
		"young adult")
}),
-- >>>>>>>>>> Alignments checked up to here.
[  "becoming part of ETC"                              ] = default({

}),
[  "becoming a famous engineer"                        ] = default({
  races = {
		"goblin",
		"gnome" },
}),
[  "becoming the next leader of %xir race"             ] = default({

}),
[  "becoming the next Warchief"                        ] = default({
  races = {
		"blood elf",
		"goblin",
		"half elf",
		"half orc",
		"orc",
		"tauren",
		"troll",
		"forsaken" },
}),
[  "becoming a great story teller"                     ] = default({

}),
[  "becoming a famous musician"                        ] = default({

}),
[  "becoming immortal"                                 ] = default({

}),
[  "becoming the HearthStone world champion"           ] = default({

}),
[  "becoming the leader of the Kirin Tor"              ] = default({
  races = {
		"blood elf",
		"gnome",
		"half elf",
		"half orc",
		"high elf",
		"human" },
  faiths = {
		"the arcane",
		"the kirin tor" },
}),
[  "becoming the leader of %xir group"                 ] = default({

}),
[  "being in control"                                  ] = default({

}),
[  "being a dancing master"                            ]	= default({
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "being a hero"                                      ] = default({

}),
[  "being a good sister"                               ] = default({ 
  genders = {
		"female" },
}),
[  "being a good brother"                              ] = default({
  genders = { 
		"male" },
}),
[  "being a good parent"                               ] = default({
	ages = npcg.data.age.agesBetween(
		"young adult",
		"adult")
}),
[  "being the top student"                             ] = default({
	ages = npcg.data.age.agesBetween(
		"teen",
		"young adult")
}),
[  "being a leading inventor"                          ] = default({
  races = { 
		"goblin", 
		"gnome" },
}),
[  "being the first to discover a new metal"           ] = default({
  classes = {
		"miner" },
}),
[  "being famous"                                      ] = default({

}),
[  "being prepared"                                    ]	= default({

}),
[  "breaking %xir addiction for bloodthistle"          ] = default({
  races = {
		"blood elf",
		"half elf",
		"high elf" },
	ages = npcg.data.age.olderThan(
		"adult")
}),
[  "breaking %xir addiction for love"                  ] = default({
	ages =
    npcg.data.age.olderThan(
		"young adult")
}),
[  "breaking %xir brother's addiction of alcohol"      ] = default({
  races = npcg.data.race.allRacesExcept(
		"dwarf" ),
}),
[  "breaking %xir addiction to fel"                    ] = default({
  races = {
		"blood elf" }, 
	ages = npcg.data.age.olderThan(   "young adult")
}),
[  "breaking %xir addiction to magic"                  ] = default({
  races = {
		"blood elf",
		"high elf" }, 
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "buying a cask of wine"                             ]	= default({
  races = {
		"blood elf",
		"dwarf",
		"gnome",
		"goblin",
		"half elf",
		"half orc",
		"high elf",
		"human",
		"night elf",
		"pandaren",
		"worgen" },
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "buying a house"                                    ] = default({
  races = npcg.data.race.allRacesExcept(
		"troll" ), 
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "buying a bed"                                      ] = default({
  races = npcg.data.race.allRacesExcept(
  		"orc",
  		"tauren",
  		"troll",
  		"forsaken" ), 
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "buying a pet"                                      ] = default({

}),
[  "buying new shoes"                                  ] = default({
  races = npcg.data.race.allRacesExcept(
  		"draenei",
  		"pandaren",
  		"tauren" ),
}),
[  "buying a pretty dress"                             ] = default({
  genders = {
		"female",
		"androgynous" },
}),
[  "buying food for %xir family"                       ] = default({
  races = {     
		"troll",
		"forsaken" },
	ages = npcg.data.age.olderThan(
		"teen")
}),  --exclude classes that dont -buy- food
[  "buying a new sword"                                ] = default({
  races = npcg.data.race.allRacesExcept(
		"forsaken" ),
  classes = { 
		"adventurer",
		"bodyguard",
		"knight",
		"mercenary",
		"paladin",
		"ranger",
		"soldier",
		"swordsman" },
}),  --sword using classes?
[  "buying a new door"                                 ] = default({

}),
[  "buying slaves"                                     ] = default({
  races = {
		"goblin",
		"orc",
		"troll",
		"forsaken" },
}),  --add slave holding people in the race,class,faith things
[  "buying bloodthistle"                               ] = default({
  races = {
		"blood elf",
		"half elf",
		"high elf" },
}),
[  "buying medicine for %xir child"                    ] = default({
	ages =
    npcg.data.age.olderThan(
		"young adult")
}),
[  "buying candy"                                      ] = default({
	ages = npcg.data.age.agesBetween(
		"child",
		"teen")
}),
[  "buying toys"                                       ] = default({
	ages = npcg.data.age.agesBetween(
		"child",
		"child")
}),
-- c ---------------------------------------------------------------------
[  "constructing a shredder"                           ]	= default({
  races = {
		"dwarf",
		"gnome",
		"goblin" },
  classes = {
		"engineer" },
}),
[  "crafting a sword"                                  ]	= default({
  classes = {
		"blacksmith" },
}),
[  "crafting a bow"                                    ]	= default({
  classes = {
		"armourer",
		"bowyer",
		"farstrider",
		"marksman",
		"ranger" },
}),
[  "crafting a paper zeppelin"                         ] = default({
	ages = npcg.data.age.agesBetween(
		"child",
		"teen")
}),
[  "crafting a siege engine"                           ] = default({
  races = {
		"dwarf",
		"gnome",
		"goblin" },
  classes = {
		"engineer" },
}),
-- d ---------------------------------------------------------------------
[  "dating a %randomRace"                              ] = default({
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "desperately wants to lift a curse"                 ] = default({
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "destroying the Legion"                             ]	= default({
  classes = {"demon hunter" },
}),
[  "disproving a magical theory"                       ] = default({
  classes = npcg.data.class.classesByType(
		"magic")
}),
[  "doing \"da Voodoo\" with a deranged Troll"         ] = default({
	ages = npcg.data.age.olderThan(
			"teen")
}),
[  "drive a trike"                                     ] = default({
  races = {
		"goblin" },
}),
[  "destroying Stormwind"                              ] = default({
  races = {
		"goblin" },
}),
[  "destroying Ironforge"                              ] = default({
  races = {
		"goblin" },
}),
[  "destroying %xir brothers relationship"             ] = default({
  races = {
		"goblin" },
}),
[  "destroying a building"                             ] = default({
  races = {
		"goblin" },
}),
[  "destroying the Tauren"                             ] = default({
  races = { 
	"goblin" },
}),
--["destroying the legion"]                         			  = { races = {		"goblin" }, classes = {     		"any" }, faiths = {                  		"any" }, genders = {     		"any" }, ages = npcg.data.age.olderThan(     		"child" ) }
[  "destroying all life on Azeroth"                    ] = default({
  races = { 
		"goblin" },
}),
[  "destroying Dalaran"                                ] = default({
  races = {
		"goblin" },
}),
[  "destroying Orgrimmar"                              ] = default({
  races = {
		"goblin" },
}),
-- e ---------------------------------------------------------------------
[  "extreme faith in %xir religion"                    ] = default({
  faiths = npcg.data.faith.allFaithsExcept(
  		"undecided",
  		"%xir people",
  		"%ximself")
}),
[  "expand others knowledge of the Arcane"             ] = default({
  faiths = { 
		"the arcane",
		"the kirin tor" },
	ages = npcg.data.age.olderThan(
		"child")
}),
[  "expand others knowledge of the Light"              ] = default({
  faiths = {
		"church of the holy light",
		"the holy light",
		"the light",
		"the order of the silver hand" },
}),
--["expand others knowledge of the twisting nether"]       = { races = {   		"any" }, classes = {     		"any" }, faiths = {                  		"any" }, genders = {     		"any" }, ages = npcg.data.age.olderThan(     		"child" ) }  
[  "expand others knowledge of history"                ] = default({

}),
[  "expand others knowledge of murlocs"                ] = default({

}),
[  "expand others knowledge of Fel"                    ] = default({

}),
[  "expand others knowledge of elven traditions"       ] = default({
  races = {
		"blood elf",
		"high elf",
		"night elf" },
}),
-- f ---------------------------------------------------------------------
[  "free %xir family member from imprisonment"         ] = default({

}),
[  "free the Old Gods"                                 ] = default({
  faiths = { 
		"the old gods" },
}),
[  "free %xir brother"                                 ] = default({

}), 
[  "free %xir sister"                                  ] = default({

}), 
[  "free %xir mind"                                    ] = default({

}), 
[  "free all animals in captivity"                     ] = default({
  classes = { 
		"druid" },
}),
[  "free others from themself"                         ] = default({
  classes = {
		"cultist",
		"doomsayer",
		"priest" },
  faiths = {
		"blood knights",
		"church of the holy light",
		"the holy light",
		"the light",
		"the old gods",
		"the void" },
}),
[  "free %xir friend from jail "                       ] = default({

}), 
[  "free Azeroth of all demons"                        ] = default({

}), 
[  "free Azeroth of orcs"                              ] = default({
  races = npcg.data.race.allRacesExcept(
		"orc")
}),
[  "figuring out who %xhe really is"                   ] = default({

}),
[  "finding a ghost crab"                              ] = default({

}),
[  "finding a job"                                     ] = default({

}),
[  "finding a place to belong"                         ] = default({

}),
[  "finding a sparring partner"                        ] = default({

}),
[  "finding out what happened to %xir baby sister"     ] = default({

}),
[  "finding out what happened to %xir brother"         ] = default({

}),
[  "finding out what happened to %xir parents"         ] = default({

}),
[  "finding %xir lost teddy"                           ] = default({
	ages = npcg.data.age.agesBetween(
		"child",
		"teen")
}),
[  "finding treasure"                                  ] = default({

}),
[  "finding %xir place in life"                        ] = default({

}),
-- g ---------------------------------------------------------------------
[  "getting a drink, or two, or three" 								] = default({
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "getting away from someone"                         ] = default({

}),
[  "getting away from the law"                         ] = default({
  classes = {
		"assassin",
		"pickpocket",
		"swindler",
		"thief",
		"outlaw",
		"thug",
		"smuggler",
		"burglar" },
}), -- Ted, add more classes in the string
[  "getting help from a medic or healer"               ] = default({

}),
[  "getting laid"                                      ] = default({
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "getting promoted"                                  ] = default({ 
	ages = npcg.data.age.olderThan(
		"teen")
}),
[  "getting adopted"                                   ] = default({
	ages = npcg.data.age.agesBetween(
		"child",
		"teen")
}),
[  "getting rich"                                      ] = default({

}),
[  "going on an adventure"                             ] = default({

}),
[  "greatly desires to kill %xir sister"               ] = default({

}),
[  "greed and anger"                                   ] = default({
  faiths = {
		"profit",
		"the old gods",
		"the void",
		"the burning legion",
		"demonology" },
}),
-- h ---------------------------------------------------------------------
[  "having a sex change"                               ] = default({
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "having a boy"                           					  ] = default({
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "having a girl"                           				  ] = default({
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "having children"                                   ] = default({

}),
[  "humiliating %xir leader"                           ] = default({

}),
-- i ---------------------------------------------------------------------
[  "inventing something special"                       ] = default({

}),
[  "inventing a new machine"                           ] = default({
  classes = {
		"engineer" },
}),
[  "inventing a mechanical butterfly"                  ] = default({

}),
[  "inventing a new way to bake cake"                  ] = default({

}),
[  "inventing a better trike"                          ] = default({
  races = {
		"goblin" },
}),
[  "inventing a cure for the Plague"                   ] = default({

}),
[  "inventing a way to make more gold"                 ] = default({
  races = {
		"goblin" },
}),
[  "inventing the best way to distribute ale"          ] = default({

}),
-- j ---------------------------------------------------------------------
[  "joining a tavern brawl"                            ] = default({

}),
[  "joining the League of Explorers"                   ] = default({

}),
[  "joining the city guards" 													  ] = default({

}),
[  "joining a group of adventurers" 										] = default({

}),
[  "joining the rebels" 																] = default({

}),
-- k ---------------------------------------------------------------------
[  "knowing all the gossip"                            ] = default({

}),
[  "knowing everything about the Light"                ] = default({

}),
[  "knowing how to raise dragons"                      ] = default({

}),
--[  "knowing when to speak and when to stay quiet"      ] = default({   }
[  "knowing how to cast a frostbolt"                   ] = default({

}),
-- l ---------------------------------------------------------------------
[  "learning a profession"                             ] = default({

}),
[  "learning how to fly"                               ] = default({

}),
[  "learning how to ride"                              ] = default({

}),
[  "living in luxury"                                  ] = default({

}),
[  "learning to control %xir temper"                   ] = default({

}),
[  "learning to defend %xir family" 										] = default({

}),
[  "learning to use a bow" 										        ] = default({

}),
[  "learning to use a sword" 										      ] = default({

}),
[  "learning how to read" 										          ] = default({

}),
[  "learning how to pickpocket" 										    ] = default({

}), --Ted, do class thing thing here
[  "learning to use magic" 										        ] = default({

}),
[  "learning how to cook" 										          ] = default({

}),
[  "learning a new language" 										      ] = default({

}),
[  "learning about other races" 										    ] = default({

}),
[  "learning about other religions" 										] = default({

}),
[  "learning how to fight" 										        ] = default({

}),
[  "learning selfdefense" 										          ] = default({
  classes = npcg.data.class.allClassesExcept(unpack(npcg.data.class.classesByType(
		"fighter" ))),
}),
-- m ---------------------------------------------------------------------
[  "making friends"                                    ] = default({

}),
--[  "making people see that we're no strangers to love" ] = default({   }
[  "mastering the fireball spell"                      ] = default({
  classes = {
		"mage" },
}), 
[  "mastering the arcane arts"                         ] = default({
  classes = {
		"mage" },
}), 
[  "mastering swordfighting"                           ] = default({
  classes = {
		"mage" },
}), 
[  "mastering archery"                                 ] = default({
  classes = {
		"mage" },
}),
[  "mastering cooking"                                 ] = default({
  classes = {
		"mage" },
}), 
[  "mastering the way of the monks"                    ] = default({
  classes = {
		"mage" },
}), 
[  "mastering poledancing"                             ] = default({
  classes = {
		"mage" },
}), 
[  "mastering the language of murlocs"                 ] = default({
  classes = {
		"mage" },
}), 
[  "mastering an ancient magic"                        ] = default({
  classes = {
		"mage" },
}), 
[  "mastering frost magic"                             ] = default({
  classes = {
		"mage" },
}), 
[  "mastering the bagpipe"                             ] = default({
  classes = {
		"mage" },
}), 
[  "mastering the art of seduction"                    ] = default({
  classes = {
		"mage" },
}), 
[  "mastering the art of brewing"                      ] = default({
  races = {
		"pandaren",
		"dwarf" },
}),
-- n ---------------------------------------------------------------------
[  "no longer losing the game"                         ] = default({

}),	-- Thank you ._.
[  "need for company"                                  ] = default({

}),
-- o ---------------------------------------------------------------------
[  "obtaining a doll" 																	] = default({
	ages = npcg.data.age.agesBetween(
		"child", "child") }),
[  "obtaining a mount"                                 ] = default({

}),
[  "obtaining a torture victim" 						 						] = default({
  faiths = {
		"the old gods", 
		"veritatem" },
}),
[  "overcoming %xir fear of %fear"                     ] = default({

}),
[  "overcoming %xir addiction to blood"                ] = default({

}),
[  "overcoming %xir addiction to magic"                ] = default({

}),
-- p ---------------------------------------------------------------------
[  "protecting the peace of the area"                  ] = default({

}),
[  "protecting %xir people"                            ] = default({

}),
[  "protecting %xir village"                           ] = default({

}),
[  "protecting %xir brother"                           ] = default({

}),
[  "protecting %xir sister"                            ] = default({

}),
[  "protecting %xir child"                             ] = default({
  ages = npcg.data.age.agesBetween(
		"young adult",
		"adult" )
}),
[  "protecting %xir parents"                           ] = default({
  ages = npcg.data.age.agesBetween( "child", "adult") }),
[  "protecting the innocent"                           ] = default({

}),
[  "protecting the cult"                               ] = default({
	classes = {
		"cultist",
		"doomsayer" },
}), --add religions to this string
[  "protecting %xir tribe"                             ] = default({
	races = npcg.data.race.racesByType(
		"tribal"),
}), --add races with tribes to this string
[  "proving a magical theory"                          ] = default({
	classes = {
		"mage" },
}),
[  "proving the Old Gods are right"                    ] = default({
	faiths = {
		"the old gods" },
}),
[  "proving that Fel is superiour to Arcane"           ] = default({
	classes = {
		"warlock" },
}),
[  "proving %xhe is right"                             ] = default({

}),
[  "proving bows are better than guns"                 ] = default({

}),
[  "proving guns are better than bows"                 ] = default({

}),
[  "proving rangers are best"                          ] = default({
	classes = {
	"ranger" },
}),
[  "proving %xhe is the best"                          ] = default({

}),
[  "proving it's safe to use an Iron Golem"            ] = default({

}),
[  "proving that inventions are meant to explode"      ] = default({
	races = {
		"goblin" },
	classes = {
		"engineer" },
}),
[  "proving %xir religion is the truth"                ] = default({

}),
[  "proving guards aren't unintelligent"               ] = default({
	classes = {
		"guard" },
}),
[  "proving magic is a faux"                           ] = default({

}),
[  "proving Azeroth doesn't exist"                     ] = default({

}),
-- q ---------------------------------------------------------------------
-- r ---------------------------------------------------------------------
[  "recovering from a knee injury"                     ] = default({
	classes = {
		"guard" },
}),
[  "retrieving a stolen heirloom"                      ] = default({

}),
[  "ruling the world"                                  ] = default({

}),
[  "ressurecting the Old Gods"                         ] = default({ 
	faiths = {
		"the old gods" },
}),
[  "ressurecting %xir loved one"                       ] = default({
  ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "ressurecting %xir child"                           ] = default({
	ages = npcg.data.age.olderThan(
		"young adult")
}),
[  "ressurecting %xir sister"                          ] = default({

}),
[  "ressurecting %xir brother"                         ] = default({

}),
-- s ---------------------------------------------------------------------
[  "scaring people"                                    ] = default({
	ages = npcg.data.age.youngerThan(
		"young adult" )
}),
[  "seeing the world"                                  ] = default({

}),
[  "spreading %xir religion"                           ] = default({

}),
[  "starting %xir own business"                        ] = default({

}),
[  "soothing %xir thirst for blood"                    ] = default({

}), 
[  "soothing %xir thirst for blood thistle"            ] = default({
	races = {
		"blood elf",
		"half elf",
		"high elf" },
}),
[  "soothing %xir thirst for alcohol"                  ] = default({

}), 
[  "solving the box of Yogg-Saron"                     ] = default({

}), 
[  "solving the mystery of life"                       ] = default({

}), 
[  "solving the answer to life"                        ] = default({

}), 
[  "starting again, somewhere else"                    ] = default({
	ages = npcg.data.age.olderThan(
		"teen")
}),
-- t ---------------------------------------------------------------------
[  "teaching others the \"true way\""                  ] = default({
  faiths = npcg.data.faith.allFaithsExcept(
		"undecided",
		"%xir people"),
}),
[  "telling stories"                                   ] = default({

}),
[  "teaching others about nature"                      ] = default({
	classes = {
		"druid" },
}),
[  "teaching %xir comrade a lesson"                    ] = default({

}),
-- u ---------------------------------------------------------------------
-- v ---------------------------------------------------------------------
[  "visiting the armoury"                              ] = default({
	classes = {
		"soldier" },
}),
[  "visit %xir parents"                                ] = default({
	ages = npcg.data.age.agesBetween(
		"teen",
		"adult")
}),
[  "visit %xir birth village"                          ] = default({

}), 
[  "visit an old friend"                               ] = default({

}), 
[  "visit %xir child"                                  ] = default({
	ages = npcg.data.age.agesBetween(
		"teen",
		"adult")
}),
-- w ---------------------------------------------------------------------
[  "wanting to solve an ancient mystery"               ] = default({

}),
[  "winning a beauty contest"                          ] = default({
	races = {
	"blood elf" },
	genders = {
		"male" },
}),
[  "winning a game of HearthStone"                     ] = default({

}),
[  "wishing to marry a hedgehog"                       ] = default({

}),
[  "wrestling a murloc"                                ] = default({

}),
[  "witness the destruction of the world"              ] = default({
 
}),
-- x ---------------------------------------------------------------------
-- y ---------------------------------------------------------------------
-- z ---------------------------------------------------------------------
} -- End of Motivations List

---------------------------------------------------------------------------------------------------------------------------------------
-- | Functions for Generation and Retrieval | -----------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

-- Main generator should be CharacterTemplate -> { Trait }
-- This will later change to be CharacterTemplate -> { Trait, weight } or something similar.

-- Part 1: Get potential candidates
-- Part 2: Weed out unfeasable candidates, like those not within context or faction
-- Part 3: Select a random item

---------------------------------------------------------------------------------------------------------------------------------------
---- | Part 1 Functions: Generation | -------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

-- These functions should be of the type:
-- characterTemplate -> zone -> subZone -> { class, classconstraints }

-- If we change the way zones and subzones work in the dataset, and make them a tree, we can abstract these functions.

local function dontCheckLocations(characterTemplate)
  return
    -- Fallback to default.
    motivations
    -- Fallback in case some extreme breaking happened.
    or { }
end

---------------------------------------------------------------------------------------------------------------------------------------
---- | Part 2 Functions: Filtering | --------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

-- These functions should be of type:
-- characterTemplate -> { class, classconstraints } -> zone -> subzone -> bool

-- We can rework this one to take in a dependency list.

local function meetsTraitConstraints(characterTemplate, option)
	return  functua.table.containsAny(option.races,   		{ "any", characterTemplate.race   		})
    and   functua.table.containsAny(option.ages,    		{ "any", characterTemplate.age    		})
    and   functua.table.containsAny(option.classes, 		{ "any", characterTemplate.class  		})
		and   functua.table.containsAny(option.genders, 		{ "any", characterTemplate.gender 		})
    and   functua.table.containsAny(option.faiths,  		{ "any", characterTemplate.faith  		})
		and   functua.table.containsAny(option.alignments,  { "any", characterTemplate.alignment  })
end

---------------------------------------------------------------------------------------------------------------------------------------
---- | Part 3 Functions: Selection | --------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

-- These functions should be of type:
-- Arbitrary a => { class, a } -> class
-- This might change in the future.

local function randomKeyFrom(options)
  return functua.random.randomFrom(functua.table.keys(options))
end

local function threeUniqueKeysFrom(options)
	return functua.random.uniqueRandomsFrom(3, functua.table.keys(options))
end

local function selectUsingWeightedRandom(selectionList)
  return functua.random.weightedRandomFrom(functua.table.keys(selectionList), function(item) return selectionList[item].weight end)
end

---------------------------------------------------------------------------------------------------------------------------------------
--| Combination Function | ------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------

-- createClassGenerator :: Trait a, TraitConstraints b =>
--	(characterTemplate -> zone -> subZone -> { a, b }) -> 
--	(characterTemplate -> { a, b } -> zone -> subzone -> bool) -> 
--	(Arbitrary a => { a, c } -> a)
--	-> (characterTemplate, zone, subZone) -> a

function createMotivationGenerator(generatorF, filterF, selectorF)
  return function(characterTemplate, zone, subZone)
    return selectorF(
      functua.table.filterV( -- Returns KVP, filters on V.
        function(potentialTrait) return filterF(characterTemplate, potentialTrait, zone, subZone) end,  -- Most don't use zone or subzone here.
        generatorF(characterTemplate, zone, subZone)
      )
    )
  end
end

generateRandomMotivation  = createMotivationGenerator(dontCheckLocations, meetsTraitConstraints, randomKeyFrom)
generateRandomMotivations = createMotivationGenerator(dontCheckLocations, meetsTraitConstraints, threeUniqueKeysFrom)

--[[
-- generateRandomMotivation :: (Race a, Class a, Faith a, Gender a, Alignment a, Age a) => Motivation
function generateRandomMotivation(characterTemplate)
  return --fixGender(characterTemplate.getGender(),
    util.randomFrom(
      functua.filter(function(item) return not functua.contains(string.lower(item), functua.map(string.lower, characterTemplate.motivations)) end,
      functua.filterKeysByValue(
        function(val) return
          (functua.contains(characterTemplate:getRace(),      val.races)      or functua.contains("any", val.races))
          and
          (functua.contains(characterTemplate:getClass(),     val.classes)    or functua.contains("any", val.classes))
          and
          (functua.contains(characterTemplate:getFaith(),     val.faiths)     or functua.contains("any", val.faiths))
          and
          (functua.contains(characterTemplate:getGender(),    val.genders)    or functua.contains("any", val.genders))
          and
          (functua.contains(characterTemplate:getAlignment(), val.alignments) or functua.contains("any", val.alignments))
          and
          (functua.contains(characterTemplate:getAge(),       val.ages)       or functua.contains("any", val.ages))
        end,
        motivations)
        )
    ) or "Error: No options found."
  --)
end
]]

--[[
-- generateRandomMotivations :: (Race a, Class a, Faith a, Gender a, Age a) => { Motivation }
function generateRandomMotivations(characterTemplate)

  local d = date("*t")
  if d.month == 4 and d.day == 1 then return
  { 
    "getting rid of the Left Shark",
    "not be killed by D.E.H.T.A",
    "wishing everyone a happy April Fools\' Day"
  }
  else
    
  return --map(function(x) return fixGender(characterTemplate.getGender(), x) end,    
    util.uniqueRandomsFrom(3, 
      functua.filterKeysByValue(
        function(val) return
          (functua.contains(characterTemplate:getRace(), val.races)           or functua.contains("any", val.races))
          and
          (functua.contains(characterTemplate:getClass(), val.classes)        or functua.contains("any", val.classes))
          and
          (functua.contains(characterTemplate:getFaith(), val.faiths)         or functua.contains("any", val.faiths))
          and
          (functua.contains(characterTemplate:getGender(), val.genders)       or functua.contains("any", val.genders))
          and
          (functua.contains(characterTemplate:getAlignment(), val.alignments) or functua.contains("any", val.alignments))
          and
          (functua.contains(characterTemplate:getAge(), val.ages)             or functua.contains("any", val.ages))
        end,
        motivations
      )
    )
end -- April fool end
  --)
end
]]

function tests.testForDuplicateMotivations()
print "Testing for duplicate motivations in npcG"
functua.iterate(
  function()
    local results = { }
    local motivs = generateRandomMotivations(GetCurrentCharacterTemplate())
    results[motivs[1]]            = (results[motivs[1]           ] or 0) + 1
    results[motivs[2] or "nil"]   = (results[motivs[2] or "nil"  ] or 0) + 1
    results[motivs[3] or "empty"] = (results[motivs[3] or "empty"] or 0) + 1
    
    if functua.any(function(x) return x > 1 end, results) then
      print "Duplicates found"
      util.logTable(motivs)
    end
  end,
  1,
  10000)
print "Testing finished"
end

function tests.testForDuplicateMotivations2()
print "Testing for duplicate motivations in npcG for the roll button"
local motivs = GetCurrentCharacterTemplate().motivations
functua.iterate(
  function()
    local results = { }
    motivs[1] = generateRandomMotivation(GetCurrentCharacterTemplate())
    results[motivs[1]]            = (results[motivs[1]           ] or 0) + 1
    results[motivs[2] or "nil"]   = (results[motivs[2] or "nil"  ] or 0) + 1
    results[motivs[3] or "empty"] = (results[motivs[3] or "empty"] or 0) + 1
    
    if functua.any(function(x) return x > 1 end, results) then
      print "Duplicates found"
      functua.logTable(motivs)
    end
  end,
  1,
  10000)
print "Testing finished"
end
--[[
local i = 0
for _,_ in pairs(motivations) do
  i = i + 1
end
print(i)
]]
-- End of File.